/* file: main.c */
/*
 * Tom Laramee
 * ECE 671,  Sim #2
 * Protocol #5 - "Selective Repeat"
 *
 *
 * No time for nice header....(sorry)
 *
 * This file simulates the "Selective Reeat" protocol in charpter 4,  one of the
 * so-called "sliding windows" protocols.
 *
 * Use scripts:
 *   o  sim1.sh
 *   o  sim1.gnuplot.sh
 *   o  sim2.sh
 *   o  sim2.gnuplot.sh
 *   o  sim3.sh
 *   o  sim3.gnuplot.sh
 *
 * ...to automate the execution fothe simulation and the generation 
 * of plots...
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include<math.h>

#include "main.h"
#include "event.h"

#define SHOWREPORT     NULL  /* contents of the final report (stdout/NULL) */ 
#define SHOWPARAMETERS NULL  /* contents of the summary of net params */
#define VERBOSE        NULL  /* show  packet transmission rate        */


#define NUMNODES 2
#define node1 0
#define node2 1


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ global vars ~~~~~~~~~ */
  
unsigned int MAXEVENTS;       /* max # of events to simulate for */
unsigned int MAXPACKETS;      /* max # of packets to simulate for */

struct event *evlist = NULL; 
struct proc *qptr[] = {NULL, NULL, NULL, NULL};

tEventPtr currEvent;    /* the current event */
tEventPtr newEvent;     /* A new event */


    FILE * fEvent;       /* rather large event list file                */
   short DEBUGGING=0;    /* debugging mode - specificed on command line */
   short done;           /* flag to stop simulation                     */

/* ~~~~~~~ variable names are from Tanenbaum book - page 240 ~~~~~~~~~ */

     int A,      /* number of bits in an ACK frame  */
         l,      /* channel length in frames        */
         D,      /* the number of data bits / frame */
         F,      /* total frame length              */
         H,      /* number of bits in frame header  */
         W,      /* window size                     */
         iseed,  /* random # seed                   */
         
         numOutstanding[NUMNODES],      /* # of unacknowledged packets         */ 
         isBusy[NUMNODES],              /* is the node busy                    */
         numInQueue[NUMNODES],          /* how many in the node's queue        */
         totalPackets[NUMNODES];        /* # of total packets transmitted      */


 double  C,   /* channel capacity in bps                          */
         E,   /* probability of a bit being in error              */
         I,   /* interrupt time + service delay                   */
         L,   /* probability that a frame is lost of damaged      */
         P1,  /* probability that a data frame is lost or damaged */
         P2,  /* probability that an ACK frame is lost or damaged */
         T,   /* timeout interval                                 */
     
        timeXPacket,    /* time to transmit a packet              */
        timeXack,       /* time to transmit an ACK                */
       
        lambda,         /* system arrival rate                    */
        sysTime,        /* global clock                           */
        timeNextEvent;  /* event times                            */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
short initParams(int argc, char * argv[])
   {
    /* This function read in all command line parameters and assigns
     * the proper values to the corresponding variables 
     */
    int counter;

    A=8;    	C=10000000;       /* default the command line parameters */
    l=1;	D=2048;
    E=0.0;	H=32;
    L=0.0;	P1=0.0;
    P2=0.0;	lambda=20;
    W=5;
 
    MAXPACKETS=100;    /* default of 100 packets for simulation */
    MAXEVENTS=1000;   
  
     /* >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> command line options */
    if (argc==1) 
      {  printf ("\nUsing the default parameters...");  return (1);  }


    for (counter=1;  counter<argc;  counter++)   /* Process args */
     if (argv[counter][0] == '-')               /* If it's a flag */
       {
       switch (argv[counter][1])
          {
           case 'A': A=atoi(argv[counter+1]);    break;
           case 'C': C=atof(argv[counter+1]);    break;
           case 'l': l=atoi(argv[counter+1]);    break;
           case 'D': D=atoi(argv[counter+1]);    break;
           case 'E': E=atof(argv[counter+1]);    break;
           case 'H': H=atoi(argv[counter+1]);    break;
           case 'L': L=atof(argv[counter+1]);     break;
           case 'p': P1=atof(argv[counter+1]);     break;
           case 'P': P2=atof(argv[counter+1]);      break;
           case 'S': lambda=atoi(argv[counter+1]);   break;
           case 'W': W=atoi(argv[counter+1]);          break;                
           case 'd': DEBUGGING=atoi(argv[counter+1]);  break;
           case 'm': MAXEVENTS=atoi(argv[counter+1]);  break;
           case 'M': MAXPACKETS=atoi(argv[counter+1]);  break;
           default :
              { printf ("\nUnrecognized command line option...(%s)(%c).\n\n", 
                     argv[counter], argv[counter][1]);
                 exit(-1);  }
          }  /* End if switch() */
       }  /* End of for loop */

  } /* End if initParams() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void initAll( void )
   {
     /* this function initializes all of the necessary variables - specifically 
      * the ones which are not on the command line and those which must be
      * computed */
   int i;

   evlist = NULL;   /* nobody's on event list now */
   done=FALSE;      /* done with simulation */

   for (i=0; i<NUMNODES; i++)
       { 
         qptr[i] = NULL;	
         numOutstanding[i]=0;      
         isBusy[i]=FALSE;
         numInQueue[i]=0;	
         totalPackets[i]=0; 
        } 
    
   F=D+H;          /* number of bits / frame */
   
   I= (double)(l*F)/C;     /* channel length = IC / F */
   
   /*  I = (F/(2.0*C))*(2.0*W + 1); */  /* to test 50% utilization */


   timeXPacket = (double)F/C;      /* packet transmission time */
   timeXack = (double)A/C;         /* ACK transmission time */
   

   T = timeXPacket + I;                 /* timeout */
        
   sysTime=0;          /* global clock */
   timeNextEvent=0;    /* the time of the next poisson event */
   iseed=1732;         /* seed the rand # generator */

   if (DEBUGGING) fEvent=fopen("p5.events", "w");
   if (DEBUGGING) if (!fEvent) fatal ("initAll", "no event file");
   }
   

/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void showParameters(FILE * fp)
  {
    /* this function simply shows the parameters of the system */ 
    if (fp != NULL)
     {
      fprintf (fp, "\nSimulation parameters: ");
      fprintf (fp, "\nDEBUGGING ?............................: %s", ((DEBUGGING==1)?"YES":"NO") );
      fprintf (fp, "\nLength of simulation in packets........: %ld", MAXPACKETS);
      fprintf (fp, "\nLength of simulation in events.........: %ld", MAXEVENTS);
      fprintf (fp, "\nWindow size............................: %d", W);
      fprintf (fp, "\nChannel length (in frames).............: %d", l);
      fprintf (fp, "\nProbability of a data frame being lost.: %lf", P1);
      fprintf (fp, "\nProbability of an ACK frame being lost.: %lf", P2);
      fprintf (fp, "\nChannel capacity (in bps)..............: %.0lf", C);
      fprintf (fp, "\nBits per ACK...........................: %d", A);
      fprintf (fp, "\nBits per header........................: %d", H);
      fprintf (fp, "\nData bits per frame....................: %d", D);
      fprintf (fp, "\nTotal bits per frame...................: %d", F);
      fprintf (fp, "\nInterrupt and service time.............: %lf", I);
      fprintf (fp, "\nTime to transmit a packet..............: %lf", timeXPacket); 
      fprintf (fp, "\nTime to transmit an ACK................: %lf", timeXack);
      fprintf (fp, "\n");
     }
  }  /* End of showParameters() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void showReport( FILE * fp )
  {
   /* This function generates the report at the end of the simulation */
   FILE * fp1;
   FILE * fp2;
   double utilization[NUMNODES];
   double avgUtilization;

                                  /* average utilization between the 2 nodes */
   utilization[node1] = 0.8*((totalPackets[node1] * timeXPacket) / sysTime);
   utilization[node2] = 0.8*((totalPackets[node2] * timeXPacket) / sysTime);

   avgUtilization=MAX(utilization[node1], utilization[node2]);


             /* generate stuff for"utilization vs channel length" plot */
   fp1=fopen("a", "w");    if (!fp1) fatal("showReport", "no fp1 file");
   fprintf (fp1, "\n%d:%lf",  l, avgUtilization);  
   fclose(fp1);

             /* generate stuff for"utilization vs window size" plot */
   fp2=fopen("b", "w");    if (!fp2) fatal("showReport", "no fp2 file");
   fprintf (fp2, "\n%d:%lf",  W, avgUtilization);  
   fclose(fp2);

   if (fp!=NULL)
    {
    fprintf (fp, "\nSimulation summary:");
    fprintf (fp, "\n> Window size............................: %d", W);
    fprintf (fp, "\n> Channel length.........................: %d", l);
    fprintf (fp, "\n> Probability of a data frame being lost.: %lf", P1);
    fprintf (fp, "\n> Probability of an ACK frame being lost.: %lf", P2);
    fprintf (fp, "\n> Num packets transmitted by node 1......: %d", totalPackets[node1]);
    fprintf (fp, "\n> Num packets transmitted by node 2......: %d", totalPackets[node2]);
    fprintf (fp, "\n> Total simulation time..................: %lf", sysTime);
    fprintf (fp, "\n> Utilization............................: %lf", avgUtilization);
    fprintf (fp, "\n");
    }

  }   /* End of showReport() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void showEventList(FILE * fp)
  {
  /* This function shows all of the events which are currently 
   * on the event list */
  tEventPtr q=NULL;
 
  if (!evlist) fatal("showEventList", "No events on the list");

  if (fp != NULL)
    {
      fprintf (fp, "\n\nEvent list for system time: %1.8lf", sysTime);
      fprintf (fp, "\nEVtime       EVcode           NODEcode  status"); 
      
      for (q=evlist;  q!=NULL; q=q->next)
        {
          fprintf (fp, "\n%1.8lf   %s    %2d       %2d",   
            q->evtime,  printEVENT(q->evcode),  q->nodecode,  q->status);
        }

    }   /* End of  if (fp !=NULL) */

  } /* End of showEventList */



/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void showCurrentEvent(FILE * fp)
  {
  /* This function shows the current event being processed */
  tEventPtr q=NULL;
 
  if (currEvent==NULL) fatal("showCurrentEvent", "No event to show");
  q=currEvent;

  if (fp != NULL)
    {
      fprintf (fp, "\nCurrent event for system time: %1.8lf", sysTime);
      fprintf (fp, "\nEVtime       EVcode           NODEcode   status");
  
      fprintf (fp, "\n%1.8lf   %s    %2d       %2d",   
           q->evtime, printEVENT(q->evcode),  q->nodecode,  q->status );
        
    }   /* End of  if (fp !=NULL) */

  } /* End of showCurrentEvent */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void printQueue( int node )
  {
    /* this function prints out the queue for a particular node */
  tProcPtr p;
  
  if (qptr[node] != NULL)
    for (p=qptr[node]; p!=NULL; p=p->next)
       fprintf (fEvent, "\nQueue %d: %1.8lf", node, p->sys_arrival_time);
   
  } /* End of printQueue() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void gen_arrival(int node)
  {
   /* This function assigns a time to an arrival based on the nodal 
    * arrival rate and the poisson assumption.  It also fills the 
    * required fields of the structure  
    */
   double timeArrival;
     
   nextarrival();                    /* Calculate a poisson arrival time */
   timeArrival = timeNextEvent;      /* Assign arrival time (poisson)    */
   newEvent = alloc_evrec();         /* Allocate room for an event       */
   newEvent->evtime = timeArrival;   /* Set it's time to occur randomly  */
   newEvent->evcode = FrameArrival;  /* Set the type of event            */
   newEvent->nodecode = node;        /* Attach ownership to a node       */
   newEvent->status=NEW;             /* it is new - not retransmit       */
   newEvent->next=NULL;              /* initialize pointers              */
   newEvent->prev=NULL;                                                   
   insertevent(newEvent);            /* Add the event to the event list  */
       
  } /* End of gen_arrival() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void gen_nodefree(int node)
  {
   /* this function generatd a "node free" event - which,  according to 
    * Tannenbaum,  is "At time F/C, the last bit [of the packet] has 
    * been sent." 
    */
   double timeArrival;     
                                     /* the node becomes free after the  */
                                     /* packet is done transmitting      */
   timeArrival = sysTime + timeXPacket;                                        
   newEvent = alloc_evrec();         /* Allocate room for an event       */
   newEvent->evtime = timeArrival;   /* Set it's time to occur randomly  */
   newEvent->evcode = NodeFree;      /* Set the type of event            */
   newEvent->nodecode = node;        /* Attach ownership to a node       */      
   newEvent->next=NULL;              /* initialize pointers              */
   newEvent->prev=NULL;              
   insertevent(newEvent);            /* Add the event to the event list  */
       
  } /* End of gen_arrival() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void gen_depart(int node)
  {
   /* This function generates a "departure" event.  According to Tannenbaum, 
    * this occurrs when: "At time (F/C) + I, the last bit [of the packet] 
    * has arrived at the receiver */
   double timeArrival;
     
   timeArrival = sysTime + timeXPacket + I;
   newEvent = alloc_evrec();         /* Allocate room for an event       */
   newEvent->evtime = timeArrival;   /* Set it's time to occur randomly  */
   newEvent->evcode = Departure;  /* Set the type of event            */
   newEvent->nodecode = node;        /* Attach ownership to a node       */
   newEvent->status=NEW;             /* it is new - not retransmit       */  
   newEvent->next=NULL;              /* initialize pointers              */
   newEvent->prev=NULL;                                                   
   insertevent(newEvent);            /* Add the event to the event list */
       
  } /* End of gen_arrival() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void gen_ackarrival(int node)
  {
   /* This function generates an AckArrival event */
   double timeArrival;
     
   timeArrival = sysTime + timeXack + I;                              
   newEvent = alloc_evrec();         /* Allocate room for an event       */
   newEvent->evtime = timeArrival;   /* Set it's time to occur randomly  */
   newEvent->evcode = AckArrival;    /* Set the type of event            */
   newEvent->nodecode = node;        /* Attach ownership to a node       */   
   newEvent->next=NULL;              /* initialize pointers              */
   newEvent->prev=NULL;                                        
   insertevent(newEvent);            /* Add the event to the event list */
       
  } /* End of gen_ack() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void gen_timeout(int node, short kind)
  {
   /* This function generates a TimeOut event */
   double timeArrival;
     
   timeArrival = sysTime + T;        /* Ths timeout is only T because     */
                                     /* packet has already been           */
                                     /* transmitting  for 1/2 the timeout */        
   newEvent = alloc_evrec();         /* Allocate room for an event        */
   newEvent->evtime = timeArrival;   /* Set it's time to occur randomly   */
   newEvent->evcode = TimeOut;       /* Set the type of event             */
   newEvent->nodecode = node;        /* Attach ownership to a node        */   
   newEvent->status=kind;            /* is it an ACK or DATA timeout ?    */
   newEvent->next=NULL;              /* initialize pointers               */
   newEvent->prev=NULL;                                        
   insertevent(newEvent);            /* Add the event to the event list  */
       
  } /* End of gen_timeout() */



/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void queueItUp( int node )
   {
   /* This function inserts an arrival event into the queue of 
    * the corresponding node  */
    tProcPtr pProc;     /* a scratch proc pointer       */
     
     numInQueue[node]++;              /* increment numInQ               */
    
     pProc=NULL;                      /* Put the packet into the queue     */
     pProc = alloc_procrec();         /* of the appropriate node           */   
     pProc->jobid = node;             /* Attach ownership to a node        */
     pProc->local_arrival_time=currEvent->evtime;
     pProc->sys_arrival_time = sysTime;
     pProc->next=NULL;                /* initialize pointers               */
     pProc->prev=NULL;         
     insertproc(pProc, node );        /* Add the proc to the node's queue  */
   
   } /* End of queueItUp */




/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void processFrameArrival( void )
  {
    /* this function processes the arrival of a frame to a node. 
     * if the node is free,  it will begin servicing the packet, otherwise, 
     * the packet will be queued 
     */

  int node;     /* the current node of interest */
  tProcPtr p;    /* a scratch proc pointer       */
 
  node=currEvent->nodecode;

  if (DEBUGGING) fprintf (fEvent, "\nEntered: frameArrvial event - node: %d", node);

  gen_arrival(node);   /* piggyback the next arrival */


    if ((isBusy[node]) || (numOutstanding[node]>=W)) 
     { 
       if (DEBUGGING) fprintf (fEvent, "\nQueue packet...");  
       queueItUp(node);
      }
    else
     {
      if (DEBUGGING) fprintf (fEvent, "\nService packet...");

      queueItUp(node);        /* if there is nothing in the queue, this   */
                              /* may be considered a waste...             */    
      p=alloc_procrec();      /* we could simply be adding this packet to */
      p=get_firstproc(node);  /* the queue and then pulling it right back */
      numInQueue[node]--;     /* off again */
      
      gen_nodefree(node);     /* generate "nodefree" event                */
      gen_depart(node);       /* generate departure event                 */
    
      numOutstanding[node]++; /* one more packet is out - with no ACK yet */
      isBusy[node]=TRUE;     /* the node is set to busy                  */
     }

  }  /* End of processFrameArrival() */



/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void processAckArrival( void )
  {
  /* this function processes the arrival of an ACK to a node.  If the node
   * has a packet in it's queue,  it will put the packet into service 
   */

  int node;       /* the current node of interest */
  tProcPtr p;     /* a scratch proc pointer       */
   
  node=currEvent->nodecode;

  if (DEBUGGING) fprintf (fEvent, "\nEntered: ACK arrival event - node: %d", node);

  numOutstanding[node]--;

  totalPackets[node]++;

  if (isBusy[node] != TRUE)
   {
      p=alloc_procrec();      /* we could simply be adding this packet to */
      p=get_firstproc(node);  /* the queue and then pulling it right back */
      numInQueue[node]--;     /* off again */
      gen_nodefree(node);     /* generate "nodefree" event                */
      gen_depart(node);       /* generate departure event                 */
      numOutstanding[node]++; /* one more packet is out - with no ACK yet */
      isBusy[node]=TRUE;     /* the node is set to busy                  */
   }

  if (VERBOSE != NULL) fprintf (VERBOSE, "\nPacket done, node: %1d, %3d", node, totalPackets[node]);

  }  /* End of processAckArrival() */



/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void processTimeOut( void )
  {
  /* this function proecesses a time-out event.  it simulates "Selective Repeat" 
   * by putting all of the outstanding packets into the node's queue for 
   * retransmission. 
   *
   * NOTE: This is an error.  It should only retransmit one packet.  It
   * doesn't affect the simulation results though...but it should be noted.
   */

  int node;      /* the current node of interest */
  tProcPtr p;    /* a scratch proc pointer       */
  int i;         /* counter                      */
  int kind;

  node=currEvent->nodecode;
  kind=currEvent->status;

  if (DEBUGGING) fprintf (fEvent, "\nEntered: proces timeOut event - node: %d", node);
  if (DEBUGGING) fprintf (fEvent, "\nEntered: type: %s", (kind==0)?"ACK":"DATA" );
 
  if (numOutstanding[node]>0)   /* make sure the node has a non-ACK'd packet */
    {
      if (DEBUGGING) fprintf (fEvent, "\nNumOutstanding: %d", numOutstanding[node]);

      if (isBusy[node]==FALSE)    /* if node is free - service the packet */
        {  
           if (DEBUGGING) fprintf (fEvent, "\nNode is idle - service front of queue");

           p=alloc_procrec();        /* malloc a proc record                     */
           p=get_firstproc(node);    /* get the 1st proc in the queue            */
           numInQueue[node]--;       /* dec numInQueue                           */
           gen_depart(node);         /* generate departure event                 */
           gen_nodefree(node);       /* generate "nodefree" event                */   
           numOutstanding[node]++;   /* one more packet is out - with no ACK yet */
           isBusy[node]=TRUE;        /* the node is set to busy                  */
           
           if (DEBUGGING) fprintf (fEvent, "\nNode is idle - Sel. Repeat");

           for (i=1;  i<numOutstanding[node]; i++)   /* start retransmitting all packets */
             queueItUp(node);                        /* by re-queueing them              */
         }  
      else     /* if node is not free - put all outstanding packets in the queue */
         { 
         if (DEBUGGING) fprintf (fEvent, "\nNode is busy - Sel. Repeat");

          for (i=1;  i<numOutstanding[node]; i++)   /* start retransmitting all packets */
             queueItUp(node);                       /* by re-queueing them              */ 
         }
    }
    else
      fatal ("processTimeOut", "no outstanding packets for this node ");
  

  }  /* End of processTimeOut() */



/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void processNodeFree( void )
  {
  /* When a node becomes free,  it may serve the packets in it's queue */
  int node;
  tProcPtr p;
  
  node=currEvent->nodecode;

  if (DEBUGGING) fprintf (fEvent, "\nEntered: nodeFree event - node: %d", node);

  if ( (qptr[node] != NULL) && (numOutstanding[node] <= W))
     {
      if (DEBUGGING) fprintf (fEvent, "\nService a packet on the front of the queue..");

      p=alloc_procrec();       /* malloc a proc record                     */
      p=get_firstproc(node);   /* get the 1st proc in the queue            */
      numInQueue[node]--;      /* dec numInQueue                           */      
      gen_nodefree(node);      /* generate "nodefree" event                */
      gen_depart(node);        /* generate departure event                 */  
      numOutstanding[node]++;  /* one more packet is out - with no ACK yet */
      isBusy[node]=TRUE;      /* the node is set to busy                  */
     }
  else
     {
      if (DEBUGGING) fprintf (fEvent, "\nNothing to service - wait for arrival");
      isBusy[node]=FALSE;      /* the node is set to not busy              */
     }

  }  /* End of processNodeFree() */



/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void processDeparture( void )
 {
  /* this is a departure event - whereupon the errors are generated.  
   * if no error- generate an "ACKarrival" event,  otherwise,  generate
   * a Timeout for either type of error
   */

  int node;
  tProcPtr p;
  short error;
  
  error=FALSE;  /* assume there is no error -- yet */

  node=currEvent->nodecode;

  if (DEBUGGING) fprintf (fEvent, "\nEntered: processDepart event - node: %d", node);

  
  if ( uni() < P1 )   
    {  gen_timeout(node, 1);  error=TRUE; }     /* data error */
    else
    {      
      if ( uni() < P2 ) 
        { gen_timeout(node, 0);   error=TRUE;  }       /* ack error */
        else 
        {   
         if (error==FALSE)                                /* no error */
           {  
            if (DEBUGGING) fprintf (fEvent, "\nSending ACK");
            gen_ackarrival(node);    
           }          
        } 
    }     

 } /* end of processDepart() */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void postCycleOP( void  )
   {
   /* This function is used only for debugging purposes.  It prints out summary
    * information @ the end of each event
    */

   tProcPtr p;  /* temporary proc pointer to print out queues */

    if (DEBUGGING) 
       {
        fprintf (fEvent, "\n\nPost-cycle information ");

        fprintf (fEvent, "\nBegin Queues");
        printQueue(node1);
        printQueue(node2);
        fprintf (fEvent, "\nEnd Queues");

        fprintf (fEvent, "\nNode 1 busy?: %s", (isBusy[node1]==TRUE)?"YES":"NO");
        fprintf (fEvent, "\nNode 2 busy?: %s", (isBusy[node2]==TRUE)?"YES":"NO");

        fprintf (fEvent, "\nTotal packets transmitted, node1: %d",totalPackets[node1]);
        fprintf (fEvent, "\nTotal packets transmitted, node2: %d",totalPackets[node2]);
       }
 
  } /* End of PostCycleOP */


/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
/* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
void main(int argc, char * argv[])
  {
  int eventNum=0;  /* keep track of event numbers                */


   initParams(argc, argv);            /* get command line args    */
   initAll();                         /* initialize all net parms */
   showParameters(SHOWPARAMETERS);    /* show parameters          */ 


   gen_arrival(node1);            /* Seed the 2 nodes with arrivals */
   gen_arrival(node2);


   while (!done)
     {
      if (DEBUGGING) showEventList(fEvent);  /* O/P the event list */      
      
      currEvent=get_nextevent();   /* Get the next event off of the event list */
      
      sysTime = currEvent->evtime;     /* Update the system clock */

      eventNum++;     /* keep track of the event Number */
      
      if (DEBUGGING)  fprintf (fEvent, "\n\n>>>>>> Current Event No: %d", eventNum);
      
      if (DEBUGGING) showCurrentEvent(fEvent);       /* Output the current event */

      
      /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ process events */
      switch(currEvent->evcode)
       {
        case FrameArrival:              /* Arrival of a frame to the channel */
             processFrameArrival();
             break;

        case AckArrival:   
             processAckArrival();
             break;

        case Departure:
             processDeparture();
             break;

        case NodeFree:
             processNodeFree();
             break;

        case TimeOut:
             processTimeOut();
             break;
                
        default:
            fatal("main", "unknown event to process");
           
       }  /* End of switch(currEvent->evcode) */
      /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */

      
      free_evrec(currEvent);                  /* Free up the memory */
         
      currEvent=NULL;                         /* point current event to NULL */
       
      postCycleOP();                         /* print out debugging O/P */
                                   
      if (eventNum>MAXEVENTS) done=TRUE;            /* are we done with simulation yet ? */
      if (totalPackets[node1]>=MAXPACKETS) done=TRUE;
      if (totalPackets[node2]>=MAXPACKETS) done=TRUE;

    }  /* End of while(!done) */ 


    showReport(SHOWREPORT);

    if (DEBUGGING)  fprintf (fEvent, "\n\n");    /* Close up any files that are open */
    if (DEBUGGING)  fclose(fEvent);
   
    /* printf ("\n"); */

} /* End of main */


